# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::PrivateDns::Mgmt::V2018_09_01
  #
  # The Private DNS Management Client.
  #
  class PrivateZones
    include MsRestAzure

    #
    # Creates and initializes a new instance of the PrivateZones class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [PrivateDnsManagementClient] reference to the PrivateDnsManagementClient
    attr_reader :client

    #
    # Creates or updates a Private DNS zone. Does not modify Links to virtual
    # networks or DNS records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param if_none_match [String] Set to '*' to allow a new Private DNS zone to
    # be created, but to prevent updating an existing zone. Other values will be
    # ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZone] operation results.
    #
    def create_or_update(resource_group_name, private_zone_name, parameters, if_match:nil, if_none_match:nil, custom_headers:nil)
      response = create_or_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, if_none_match:if_none_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param if_none_match [String] Set to '*' to allow a new Private DNS zone to
    # be created, but to prevent updating an existing zone. Other values will be
    # ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def create_or_update_async(resource_group_name, private_zone_name, parameters, if_match:nil, if_none_match:nil, custom_headers:nil)
      # Send request
      promise = begin_create_or_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, if_none_match:if_none_match, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Updates a Private DNS zone. Does not modify virtual network links or DNS
    # records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the Update operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZone] operation results.
    #
    def update(resource_group_name, private_zone_name, parameters, if_match:nil, custom_headers:nil)
      response = update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the Update operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def update_async(resource_group_name, private_zone_name, parameters, if_match:nil, custom_headers:nil)
      # Send request
      promise = begin_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Deletes a Private DNS zone. WARNING: All DNS records in the zone will also be
    # deleted. This operation cannot be undone. Private DNS zone cannot be deleted
    # unless all virtual network links to it are removed.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always delete the current zone. Specify the last-seen ETag value to prevent
    # accidentally deleting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def delete(resource_group_name, private_zone_name, if_match:nil, custom_headers:nil)
      response = delete_async(resource_group_name, private_zone_name, if_match:if_match, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always delete the current zone. Specify the last-seen ETag value to prevent
    # accidentally deleting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(resource_group_name, private_zone_name, if_match:nil, custom_headers:nil)
      # Send request
      promise = begin_delete_async(resource_group_name, private_zone_name, if_match:if_match, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Gets a Private DNS zone. Retrieves the zone properties, but not the virtual
    # networks links or the record sets within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZone] operation results.
    #
    def get(resource_group_name, private_zone_name, custom_headers:nil)
      response = get_async(resource_group_name, private_zone_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a Private DNS zone. Retrieves the zone properties, but not the virtual
    # networks links or the record sets within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, private_zone_name, custom_headers:nil)
      get_async(resource_group_name, private_zone_name, custom_headers:custom_headers).value!
    end

    #
    # Gets a Private DNS zone. Retrieves the zone properties, but not the virtual
    # networks links or the record sets within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, private_zone_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'private_zone_name is nil' if private_zone_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'privateZoneName' => private_zone_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param top [Integer] The maximum number of Private DNS zones to return. If
    # not specified, returns up to 100 zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<PrivateZone>] operation results.
    #
    def list(top:nil, custom_headers:nil)
      first_page = list_as_lazy(top:top, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param top [Integer] The maximum number of Private DNS zones to return. If
    # not specified, returns up to 100 zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(top:nil, custom_headers:nil)
      list_async(top:top, custom_headers:custom_headers).value!
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param top [Integer] The maximum number of Private DNS zones to return. If
    # not specified, returns up to 100 zones.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(top:nil, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Network/privateDnsZones'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id},
          query_params: {'$top' => top,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZoneListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param top [Integer] The maximum number of record sets to return. If not
    # specified, returns up to 100 record sets.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<PrivateZone>] operation results.
    #
    def list_by_resource_group(resource_group_name, top:nil, custom_headers:nil)
      first_page = list_by_resource_group_as_lazy(resource_group_name, top:top, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param top [Integer] The maximum number of record sets to return. If not
    # specified, returns up to 100 record sets.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_with_http_info(resource_group_name, top:nil, custom_headers:nil)
      list_by_resource_group_async(resource_group_name, top:top, custom_headers:custom_headers).value!
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param top [Integer] The maximum number of record sets to return. If not
    # specified, returns up to 100 record sets.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_async(resource_group_name, top:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'subscriptionId' => @client.subscription_id},
          query_params: {'$top' => top,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZoneListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates or updates a Private DNS zone. Does not modify Links to virtual
    # networks or DNS records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param if_none_match [String] Set to '*' to allow a new Private DNS zone to
    # be created, but to prevent updating an existing zone. Other values will be
    # ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZone] operation results.
    #
    def begin_create_or_update(resource_group_name, private_zone_name, parameters, if_match:nil, if_none_match:nil, custom_headers:nil)
      response = begin_create_or_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, if_none_match:if_none_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates or updates a Private DNS zone. Does not modify Links to virtual
    # networks or DNS records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param if_none_match [String] Set to '*' to allow a new Private DNS zone to
    # be created, but to prevent updating an existing zone. Other values will be
    # ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_create_or_update_with_http_info(resource_group_name, private_zone_name, parameters, if_match:nil, if_none_match:nil, custom_headers:nil)
      begin_create_or_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, if_none_match:if_none_match, custom_headers:custom_headers).value!
    end

    #
    # Creates or updates a Private DNS zone. Does not modify Links to virtual
    # networks or DNS records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param if_none_match [String] Set to '*' to allow a new Private DNS zone to
    # be created, but to prevent updating an existing zone. Other values will be
    # ignored.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_create_or_update_async(resource_group_name, private_zone_name, parameters, if_match:nil, if_none_match:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'private_zone_name is nil' if private_zone_name.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['If-None-Match'] = if_none_match unless if_none_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'privateZoneName' => private_zone_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 201 || status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Updates a Private DNS zone. Does not modify virtual network links or DNS
    # records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the Update operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZone] operation results.
    #
    def begin_update(resource_group_name, private_zone_name, parameters, if_match:nil, custom_headers:nil)
      response = begin_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Updates a Private DNS zone. Does not modify virtual network links or DNS
    # records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the Update operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_update_with_http_info(resource_group_name, private_zone_name, parameters, if_match:nil, custom_headers:nil)
      begin_update_async(resource_group_name, private_zone_name, parameters, if_match:if_match, custom_headers:custom_headers).value!
    end

    #
    # Updates a Private DNS zone. Does not modify virtual network links or DNS
    # records within the zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param parameters [PrivateZone] Parameters supplied to the Update operation.
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always overwrite the current zone. Specify the last-seen ETag value to
    # prevent accidentally overwriting any concurrent changes.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_update_async(resource_group_name, private_zone_name, parameters, if_match:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'private_zone_name is nil' if private_zone_name.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'privateZoneName' => private_zone_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:patch, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZone.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes a Private DNS zone. WARNING: All DNS records in the zone will also be
    # deleted. This operation cannot be undone. Private DNS zone cannot be deleted
    # unless all virtual network links to it are removed.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always delete the current zone. Specify the last-seen ETag value to prevent
    # accidentally deleting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_delete(resource_group_name, private_zone_name, if_match:nil, custom_headers:nil)
      response = begin_delete_async(resource_group_name, private_zone_name, if_match:if_match, custom_headers:custom_headers).value!
      nil
    end

    #
    # Deletes a Private DNS zone. WARNING: All DNS records in the zone will also be
    # deleted. This operation cannot be undone. Private DNS zone cannot be deleted
    # unless all virtual network links to it are removed.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always delete the current zone. Specify the last-seen ETag value to prevent
    # accidentally deleting any concurrent changes.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(resource_group_name, private_zone_name, if_match:nil, custom_headers:nil)
      begin_delete_async(resource_group_name, private_zone_name, if_match:if_match, custom_headers:custom_headers).value!
    end

    #
    # Deletes a Private DNS zone. WARNING: All DNS records in the zone will also be
    # deleted. This operation cannot be undone. Private DNS zone cannot be deleted
    # unless all virtual network links to it are removed.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param private_zone_name [String] The name of the Private DNS zone (without a
    # terminating dot).
    # @param if_match [String] The ETag of the Private DNS zone. Omit this value to
    # always delete the current zone. Specify the last-seen ETag value to prevent
    # accidentally deleting any concurrent changes.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(resource_group_name, private_zone_name, if_match:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'private_zone_name is nil' if private_zone_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'privateZoneName' => private_zone_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202 || status_code == 204
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZoneListResult] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZoneListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZoneListResult] operation results.
    #
    def list_by_resource_group_next(next_page_link, custom_headers:nil)
      response = list_by_resource_group_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_group_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_resource_group_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_group_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::PrivateDns::Mgmt::V2018_09_01::Models::PrivateZoneListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the Private DNS zones in all resource groups in a subscription.
    #
    # @param top [Integer] The maximum number of Private DNS zones to return. If
    # not specified, returns up to 100 zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZoneListResult] which provide lazy access to pages of the
    # response.
    #
    def list_as_lazy(top:nil, custom_headers:nil)
      response = list_async(top:top, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists the Private DNS zones within a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param top [Integer] The maximum number of record sets to return. If not
    # specified, returns up to 100 record sets.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateZoneListResult] which provide lazy access to pages of the
    # response.
    #
    def list_by_resource_group_as_lazy(resource_group_name, top:nil, custom_headers:nil)
      response = list_by_resource_group_async(resource_group_name, top:top, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_resource_group_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
